library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity clock is
	Port(
		hardware_clock : in STD_LOGIC;
		clear : in STD_LOGIC;
		count : in STD_LOGIC;
		set	: in STD_LOGIC;
		add_min : in STD_LOGIC;
		add_hour : in STD_LOGIC;
		milliseconds : out STD_LOGIC_VECTOR (15 downto 0);
		seconds : out STD_LOGIC_VECTOR (15 downto 0);
		minutes : out STD_LOGIC_VECTOR (15 downto 0);
		hours : out STD_LOGIC_VECTOR (15 downto 0);
		ampm : out STD_LOGIC
	);
end clock;

architecture Behavioral of clock is
component counter is 
	Port(
		clock : in STD_LOGIC;
		clear : in STD_LOGIC;
		count : in STD_LOGIC;
		q : out STD_LOGIC_VECTOR (15 downto 0)
		);
end component;

component Adder2 is
	Port(
		a : in STD_LOGIC_VECTOR (15 downto 0);
		b : in STD_LOGIC_VECTOR (15 downto 0);
		cin : in STD_LOGIC;

		f : out STD_LOGIC_VECTOR (15 downto 0);
		add_cout : out STD_LOGIC
		);
end component;


signal second_clock : STD_LOGIC;
signal hour_clock : STD_LOGIC;
signal minute_clock : STD_LOGIC;
signal day_clock : STD_LOGIC;
signal ap_clock : STD_LOGIC;

signal ms_time : STD_LOGIC_VECTOR (15 downto 0);
signal second_time : STD_LOGIC_VECTOR (15 downto 0);
signal minute_time : STD_LOGIC_VECTOR (15 downto 0);
signal hour_time : STD_LOGIC_VECTOR (15 downto 0);
signal ap_time : STD_LOGIC_VECTOR (15 downto 0);

begin
	-- 20 * 50ms = 1000ms, 20 in binary is 10100
	second_clock <= clear or (ms_time(9) and ms_time(8) and ms_time(7) and ms_time(6) and ms_time(5) and not ms_time(4) and ms_time(3) and not ms_time(2) and not ms_time(1) and not ms_time(0));
	-- 111100 = 60s
	minute_clock <= clear or ( set and add_min and second_clock) or (second_time(5) and second_time(4) and second_time(3) and second_time(2) and not second_time(1) and not second_time(0));
	hour_clock <= clear or (set and add_hour and second_clock) or (minute_time(5) and minute_time(4) and minute_time(3) and not minute_time(2) and minute_time(1) and minute_time(0));
	-- 11000 = 24
	day_clock <= clear or (not hour_time(4) and hour_time(3) and hour_time(2) and not hour_time(1) and not hour_time(0));
	ap_clock <= clear or ap_time(1);

	ms_component: counter
		port map (hardware_clock, second_clock, count, ms_time);
	second_component: counter
		port map (second_clock, minute_clock, count, second_time);
	minute_component: counter
		port map (minute_clock, hour_clock, count, minute_time);
	hour_component: counter
		port map (hour_clock, day_clock, count, hour_time);
	ap_component: counter
		port map (day_clock, ap_clock, count, ap_time);
		
	milliseconds <= ms_time;
	seconds <= second_time;
	minutes <= minute_time;
	
	
	
	hours <= hour_time;
	ampm <= ap_time(0);
end Behavioral;



