library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity device is
	Port(
		hardware_clock : in STD_LOGIC;
		clear : in STD_LOGIC;
		time_set : in STD_LOGIC;
		alarm_set : in STD_LOGIC;
		show : in STD_LOGIC_VECTOR (1	downto 0);
		add_min : in STD_LOGIC;
		add_hour : in STD_LOGIC;
		input : in STD_LOGIC;
		reset : in STD_LOGIC;
		led : out STD_LOGIC_VECTOR (7 downto 0);
		anode : out STD_LOGIC_VECTOR (3 downto 0);
		segment_out : out STD_LOGIC_VECTOR (6 downto 0)
	);
end device;

architecture Behavioral of device is
component clock is
	Port(
		hardware_clock : in STD_LOGIC;
		clear : in STD_LOGIC;
		count : in STD_LOGIC;
		set	: in STD_LOGIC;
		add_min : in STD_LOGIC;
		add_hour : in STD_LOGIC;
		milliseconds : out STD_LOGIC_VECTOR (15 downto 0);
		seconds : out STD_LOGIC_VECTOR (15 downto 0);
		minutes : out STD_LOGIC_VECTOR (15 downto 0);
		hours : out STD_LOGIC_VECTOR (15 downto 0);
		ampm : out STD_LOGIC
	);
end component;

component clockdivider is
    Port ( clock : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           clockout : out  STD_LOGIC);
end component;

component display is
	Port(
		display1 : in STD_LOGIC_VECTOR (15 downto 0);
		display2 : in STD_LOGIC_VECTOR (15 downto 0);
		segment0 : out STD_LOGIC_VECTOR (6 downto 0);
		segment1 : out STD_LOGIC_VECTOR (6 downto 0);
		segment2 : out STD_LOGIC_VECTOR (6 downto 0);
		segment3 : out STD_LOGIC_VECTOR (6 downto 0)
	);
end component;

signal display1 : STD_LOGIC_VECTOR (15 downto 0);
signal display2 : STD_LOGIC_VECTOR (15 downto 0);


signal set_sec : STD_LOGIC_VECTOR (15 downto 0);
signal set_min : STD_LOGIC_VECTOR (15 downto 0);
signal set_hour : STD_LOGIC_VECTOR (15 downto 0);

signal clock_ms : STD_LOGIC_VECTOR (15 downto 0);
signal clock_s : STD_LOGIC_VECTOR (15 downto 0);
signal clock_m : STD_LOGIC_VECTOR (15 downto 0);
signal clock_h : STD_LOGIC_VECTOR (15 downto 0);
signal clock_ampm : STD_LOGIC;

signal alarm_ms : STD_LOGIC_VECTOR (15 downto 0);
signal alarm_s : STD_LOGIC_VECTOR (15 downto 0);
signal alarm_m : STD_LOGIC_VECTOR (15 downto 0);
signal alarm_h : STD_LOGIC_VECTOR (15 downto 0);
signal alarm_ampm : STD_LOGIC;
signal alarm_on : STD_LOGIC;

signal watch_ms : STD_LOGIC_VECTOR (15 downto 0);
signal watch_s : STD_LOGIC_VECTOR (15 downto 0);
signal watch_m : STD_LOGIC_VECTOR (15 downto 0);
signal watch_h : STD_LOGIC_VECTOR (15 downto 0);
signal watch_ampm : STD_LOGIC;

signal segment0 : STD_LOGIC_VECTOR (6 downto 0);
signal segment1 : STD_LOGIC_VECTOR (6 downto 0);
signal segment2 : STD_LOGIC_VECTOR (6 downto 0);
signal segment3 : STD_LOGIC_VECTOR (6 downto 0);

signal temp_anode : STD_LOGIC_VECTOR (3 downto 0);
signal clocksig : STD_LOGIC;
signal count : STD_LOGIC;

signal show_clock : STD_LOGIC;
signal show_altclock : STD_LOGIC;
signal show_alarm : STD_LOGIC;
signal show_watch : STD_LOGIC;
signal show_alarm_clock : STD_LOGIC;

TYPE state IS (S0, S1, S2);
SIGNAL Moore_state: state;
signal s_clear : STD_LOGIC;
signal s_count : STD_LOGIC;
signal state0 : STD_LOGIC;
signal state1 : STD_LOGIC;
signal state2 : STD_LOGIC;
signal wcount : STD_LOGIC;
signal naltclock : STD_LOGIC;

begin
	count <= '1';
	
U_Moore: PROCESS (clock_s(0), clear)
BEGIN
	IF(reset = '1') THEN
		Moore_state <= S0;
	ELSIF (clock_s(0) = '1' AND clock_s(0)'event) THEN
		CASE Moore_state IS 
WHEN S0 =>
			  IF input = '1' THEN
	           Moore_state <= S1; 
              ELSE
              Moore_state <= S0;
        END IF;
WHEN S1 =>
			  IF input = '1' THEN 
              Moore_state <= S2; 
              ELSE
              Moore_state <= S1; 
         END IF;
WHEN S2 =>
			  IF input = '1' THEN
               Moore_state <= S0; 
					ELSE 
               Moore_state <= S2; 
          END IF;

		  END CASE;
	END IF;
END PROCESS;
state0 <= '1' WHEN Moore_state = S0 ELSE '0';
state1 <= '1' WHEN Moore_state = S1 ELSE '0';
state2 <= '1' WHEN Moore_state = S2 ELSE '0';

	wcount <= (state1 or (not state2));


	divider_component: clockdivider
		port map (hardware_clock, clear, clocksig);

	clock_component: clock
		port map (clocksig, clear, count, time_set, add_min, add_hour, clock_ms, clock_s, clock_m, clock_h, clock_ampm);

	alarm_component: clock
		port map (clocksig, clear, count, alarm_set, add_min, add_hour, alarm_ms, alarm_s, alarm_m, alarm_h, alarm_ampm);
		
	stopwatch_component: clock
		port map (clocksig, state0, wcount, not time_set, add_min, add_hour, watch_ms, watch_s, watch_m, watch_h, watch_ampm);

--	alarm_on <= (clock_m and alarm_m) and (alarm_h and clock_h);
	temp_anode(0) <= not clock_ms(0) and not clock_ms(1);
	temp_anode(1) <= clock_ms(0) and not clock_ms(1);
	temp_anode(2) <= not clock_ms(0) and clock_ms(1);
	temp_anode(3) <= clock_ms(0) and clock_ms(1);
	
	show_clock <= not show(0) and not show(1);
	show_alarm <= not show(0) and show(1);
	show_watch <= show(0) and not show(1);
	show_altclock <= show(0) and show(1);

	
	display1(0) <= (clock_m(0) and show_clock) or (clock_ms(0) and show_altclock) or (alarm_m(0) and show_alarm) or (watch_s(0) and show_watch);
	display1(1) <= (clock_m(1) and show_clock) or (clock_ms(1) and show_altclock) or (alarm_m(1) and show_alarm) or (watch_s(1) and show_watch);
	display1(2) <= (clock_m(2) and show_clock) or (clock_ms(2) and show_altclock) or (alarm_m(2) and show_alarm) or (watch_s(2) and show_watch);
	display1(3) <= (clock_m(3) and show_clock) or (clock_ms(3) and show_altclock) or (alarm_m(3) and show_alarm) or (watch_s(3) and show_watch);
	display1(4) <= (clock_m(4) and show_clock) or (clock_ms(4) and show_altclock) or (alarm_m(4) and show_alarm) or (watch_s(4) and show_watch);
	display1(5) <= (clock_m(5) and show_clock) or (clock_ms(5) and show_altclock) or (alarm_m(5) and show_alarm) or (watch_s(5) and show_watch);
	display1(6) <= (clock_m(6) and show_clock) or (clock_ms(6) and show_altclock) or (alarm_m(6) and show_alarm) or (watch_s(6) and show_watch);
	
	display2(0) <= (clock_h(0) and show_clock) or (clock_s(0) and show_altclock) or (alarm_h(0) and show_alarm) or (watch_m(0) and show_watch);
	display2(1) <= (clock_h(1) and show_clock) or (clock_s(1) and show_altclock) or (alarm_h(1) and show_alarm) or (watch_m(1) and show_watch);
	display2(2) <= (clock_h(2) and show_clock) or (clock_s(2) and show_altclock) or (alarm_h(2) and show_alarm) or (watch_m(2) and show_watch);
	display2(3) <= (clock_h(3) and show_clock) or (clock_s(3) and show_altclock) or (alarm_h(3) and show_alarm) or (watch_m(3) and show_watch);
	display2(4) <= (clock_h(4) and show_clock) or (clock_s(4) and show_altclock) or (alarm_h(4) and show_alarm) or (watch_m(4) and show_watch);
	display2(5) <= (clock_h(5) and show_clock) or (clock_s(5) and show_altclock) or (alarm_h(5) and show_alarm) or (watch_m(5) and show_watch);
	display2(6) <= (clock_h(6) and show_clock) or (clock_s(6) and show_altclock) or (alarm_h(6) and show_alarm) or (watch_m(6) and show_watch);

	display_component: display
		port map (display1, display2, segment0, segment1, segment2, segment3);
		
	led(0) <= clock_s(0);
	led(1) <= clock_s(1);
	led(2) <= clock_s(2);
	led(3) <= clock_s(3);
	led(4) <= clock_m(0);
	led(5) <= clock_m(1);
	led(6) <= clock_m(2);
	led(7) <= clock_m(3);
	
	naltclock <= not show_altclock;
		
	segment_out(0) <= (segment0(0) and temp_anode(0) and naltclock) or (segment1(0) and temp_anode(1) and naltclock) or (segment2(0) and temp_anode(2)) or (segment3(0) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(0) and show_altclock and clock_ampm) or ('0' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(1) <= (segment0(1) and temp_anode(0) and naltclock) or (segment1(1) and temp_anode(1) and naltclock) or (segment2(1) and temp_anode(2)) or (segment3(1) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(0) and show_altclock and clock_ampm) or ('0' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(2) <= (segment0(2) and temp_anode(0) and naltclock) or (segment1(2) and temp_anode(1) and naltclock) or (segment2(2) and temp_anode(2)) or (segment3(2) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('1' and temp_anode(0) and show_altclock and clock_ampm) or ('1' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(3) <= (segment0(3) and temp_anode(0) and naltclock) or (segment1(3) and temp_anode(1) and naltclock) or (segment2(3) and temp_anode(2)) or (segment3(3) and temp_anode(3)) or ('1' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('1' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('1' and temp_anode(0) and show_altclock and clock_ampm) or ('1' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(4) <= (segment0(4) and temp_anode(0) and naltclock) or (segment1(4) and temp_anode(1) and naltclock) or (segment2(4) and temp_anode(2)) or (segment3(4) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(0) and show_altclock and clock_ampm) or ('0' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(5) <= (segment0(5) and temp_anode(0) and naltclock) or (segment1(5) and temp_anode(1) and naltclock) or (segment2(5) and temp_anode(2)) or (segment3(5) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(0) and show_altclock and clock_ampm) or ('0' and temp_anode(1) and show_altclock and clock_ampm);
	segment_out(6) <= (segment0(6) and temp_anode(0) and naltclock) or (segment1(6) and temp_anode(1) and naltclock) or (segment2(6) and temp_anode(2)) or (segment3(6) and temp_anode(3)) or ('0' and temp_anode(0) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(1) and show_altclock and (not clock_ampm)) or ('0' and temp_anode(0) and show_altclock and clock_ampm) or ('0' and temp_anode(1) and show_altclock and clock_ampm);
	anode <= not temp_anode;
end Behavioral;

